#include <qapplication.h>
#include <qspinbox.h>
#include <math.h>
#include <qvalidator.h>

class FloatSpinBox : public QSpinBox {

public:

    FloatSpinBox( int min, int max, float step, int digits,
                  QWidget *parent, const char *name = 0 )
        : QSpinBox( (int) (min*pow(digits,10)), (int) (max*pow(digits,10)),
                    (int) (step*pow(digits,10)), parent, name ),
          _digits( digits )
        {
            setValue( (int) (min*pow(digits,10)) );
            delete validator();
            QDoubleValidator* validator =
                new QDoubleValidator( min,  max, _digits, this );
            setValidator( validator );
        }

    virtual QString  mapValueToText( int value ) {
        QString format = QString("%.%1f").arg( _digits );
        return QString().sprintf(format.latin1(),
				 (value/(float)pow(_digits,10)) );
    }

    virtual int mapTextToValue( bool* ok ) {
        return int(cleanText().toFloat(ok)*pow(_digits,10));
    }

private:
    int _digits;
};

int main( int argc, char** argv ) {
    QApplication app( argc, argv );

    FloatSpinBox* spin = new FloatSpinBox( 5 /*min*/, 12 /*max*/,
                                           0.05 /*step*/, 3 /*digits*/, 0 );
    spin->setPrefix( "$" );
    spin->show();

    QObject::connect( qApp, SIGNAL( lastWindowClosed() ),
		      qApp, SLOT( quit() ) );
    return app.exec();
}
