#ifndef LISTVIEWTOOLTIP_H
#define LISTVIEWTOOLTIP_H

#include <qtooltip.h>
#include <qlistview.h>
#include <qheader.h>

class ListViewToolTip : public QToolTip
{
public:
    ListViewToolTip( QListView* parent );

protected:
    void maybeTip( const QPoint& p );

private:
    QListView* listView;
};

inline ListViewToolTip::ListViewToolTip( QListView* parent )
    : QToolTip( parent->viewport() ), listView( parent ) {}

inline void ListViewToolTip::maybeTip( const QPoint& p ) {
    if ( !listView )
        return;

    const QListViewItem* item = listView->itemAt( p );
    if ( !item )
        return;

    const QRect itemRect = listView->itemRect( item );
    if ( !itemRect.isValid() )
        return;

    const int col = listView->header()->sectionAt( p.x() );
    if ( col == -1 )
        return;

    const QRect headerRect = listView->header()->sectionRect( col );
    if ( !headerRect.isValid() )
        return;

    const QRect cellRect( headerRect.left(), itemRect.top(),
                          headerRect.width(), itemRect.height() );

    QString tipStr;
    if( col == 0 )
        tipStr = QString( "Contents of this cell: <b>%1</b>" ).arg(
                              item->text( 0 ) );
    else if( col == 1 )
        tipStr = QString( "Contents of that cell: <em>%1</em>" ).arg(
                              item->text( 1 ) );
    else
        tipStr = QString( "Something else" );

    tip( cellRect, tipStr );
}


#endif /* LISTVIEWTOOLTIP_H */

