#include <qapplication.h>
#include <qsplitter.h>
#include <qtextedit.h>
#include <qobjectlist.h>
#include <qtimer.h>

class SyncSplitter :public QSplitter {
    Q_OBJECT

public:

    SyncSplitter( Orientation orient, QWidget* parent, const char* name = 0 )
        :QSplitter( orient, parent, name )  {
    }

signals:

    void resizing( QValueList<int> );

public slots:

    void setSizes( QValueList<int> sizes )  {
        int sum = 0;
        for ( uint i = 0; i < sizes.count()-1; ++i )  {
            sum += sizes[i];
            moveSplitter( sum, i*2+1 );
            sum += 6;
        }
    }

protected:

    void childEvent( QChildEvent *c )  {
        QSplitter::childEvent( c );
        if ( c->child()->className() == QCString( "QSplitterHandle" ) ) {
            c->child()->installEventFilter( this );
        }
    }

    virtual bool eventFilter( QObject*,  QEvent* e )  {
        if ( e->type() == QEvent::MouseMove )  {
            if (  opaqueResize() )
                QTimer::singleShot( 0, this, SLOT( timeout() ) );

        }
        else if ( e->type() == QEvent::MouseButtonRelease )  {
            QTimer::singleShot( 0, this, SLOT( timeout() ) );
        }
        return false;
    }

protected slots:

    void timeout()  {
        emit resizing( sizes() );
    }
};

int main( int argc, char** argv ) {
    QApplication app( argc, argv );

    QSplitter* s1 = new QSplitter( Qt::Vertical, 0 );

    SyncSplitter* s2 = new SyncSplitter( Qt::Horizontal, s1 );
    SyncSplitter* s3 = new SyncSplitter( Qt::Horizontal, s1 );
    QObject::connect( s2, SIGNAL( resizing( QValueList<int> ) ), 
		      s3, SLOT( setSizes( QValueList<int> ) ) );
    QObject::connect( s3, SIGNAL( resizing( QValueList<int> ) ), 
		      s2, SLOT( setSizes( QValueList<int> ) ) );

    QTextEdit* e1 = new QTextEdit( s2 );
    QTextEdit* e2 = new QTextEdit( s2 );
    QTextEdit* e3 = new QTextEdit( s3 );
    QTextEdit* e4 = new QTextEdit( s3 );

    // just to ensure that this also works.
    s2->setOpaqueResize ();

    e1->setText( "Edit 1" );
    e2->setText( "Edit 2" );
    e3->setText( "Edit 3" );
    e4->setText( "Edit 4" );

    s1->show();

    QObject::connect( qApp, SIGNAL( lastWindowClosed() ),
		      qApp, SLOT( quit() ) );
    return app.exec();
}

#include "main.moc"
