#ifndef SHOOTABUG_H
#define SHOOTABUG_H

#include <qevent.h>
#include <qapplication.h>
#include <qobjectlist.h>
#include <qlabel.h>
#include <unistd.h>

static void myDumpRecursive( int level, QObject *object )
{
#if 1
  if ( object ) {
    QString buf;
    buf.fill( '\t', level/2 );
    if ( level % 2 )
	    buf += "    ";
    const char *name = object->name();
    QString flags="";
    if ( qApp->focusWidget() == object )
	    flags += 'F';
    if ( object->isWidgetType() ) {
	    QWidget * w = (QWidget *)object;
	    if ( w->isVisible() ) {
        QString t( "<%1,%2,%3,%4>" );
        flags += t.arg(w->x()).arg(w->y()).arg(w->width()).arg(w->height());
	    } else {
        flags += 'I';
	    }
    }
    qDebug( "%s%p %s::%s %s", (const char*)buf, object, object->className(), name,
            flags.latin1() );
    if ( object->children() ) {
	    QObjectListIt it(*object->children());
	    QObject * c;
	    while ( (c=it.current()) != 0 ) {
        ++it;
        myDumpRecursive( level+1, c );
	    }
    }
  }
#else
  Q_UNUSED( level )
  Q_UNUSED( object )
#endif
    }

class ShootABug :public QObject
{
public:
  virtual bool eventFilter( QObject* recv, QEvent* event )
  {
    if ( event->type() == QEvent::MouseButtonPress ) {
      int state =  static_cast<QMouseEvent*>(event)->stateAfter();
      QWidget* w = static_cast<QWidget*>( recv );
      if ( (state & Qt::ControlButton ) &&
           (state & Qt::ShiftButton ) &&
           (state & Qt::LeftButton) ) {
        // Ctrl + left mouse click.

        qDebug("----------------------------------------------------");
        print( w );
        //qDebug("Geometry: local offset=(%d,%d) global pos=(%d,%d) size=%dx%d, sizeHint=%dx%d",
        //       w->x(), w->y(), w->mapToGlobal(w->pos()).x(), w->mapToGlobal(w->pos()).y(), w->width(), w->height(),
        //       w->sizeHint().width(), w->sizeHint().height());

        if ( state & Qt::AltButton ) {
          qDebug("\nObject info:");
          recv->dumpObjectInfo();
          qDebug("\nObject tree:");
          myDumpRecursive( 0, recv );
        }

        qDebug("----------------------------------------------------");
        return true; // Block
      }
      else if (  (state & Qt::ControlButton ) &&
                 (state & Qt::AltButton ) &&
                 (state & Qt::LeftButton) ) {
        blink( w );
        return true; // No propagate please.
      }
    }

    return false;
  }

  void blink( QWidget* obj )
  {
#ifdef Q_WS_X11
    for ( int i = 0; i < 16; ++i ) {
      QPalette pal = obj->palette();
      for ( int role = QColorGroup::Foreground; role < QColorGroup::NColorRoles; ++ role ) {
        for ( int which = 0; which < 3; ++ which ) {
          QColor col = pal.color( (QPalette::ColorGroup) which,
                                  (QColorGroup::ColorRole) role  );
          int r = (col.red() + 16)%256;
          int g = (col.green() + 16)%256;
          int b = (col.blue() + 16)%256;
          pal.setColor( (QPalette::ColorGroup) which,
                        (QColorGroup::ColorRole) role, QColor( r,g,b ) );
        }
      }
      obj->setPalette( pal );
      obj->repaint();
      qApp->processEvents();
      usleep( 100000 );
    }
#endif
  }






  static void print( QWidget* w  ) {
    if ( w->parent() && w->parent()->isWidgetType() ) {
      print( static_cast<QWidget*>( w->parent() ) );
    }
    qDebug( "ptr=%p class=%s name=%s local offset=(%d,%d) global pos=(%d,%d) size=%dx%d, min=%dx%d, max=%d,%d, hint=%dx%d",
            w, w->className(), w->name(), w->x(), w->y(), w->mapToGlobal(w->pos()).x(), w->mapToGlobal(w->pos()).y(),
            w->width(), w->height(), w->minimumSize().width(), w->minimumSize().height(), w->maximumSize().width(),
            w->maximumSize().height(),w->sizeHint().width(), w->sizeHint().height());
  }
};


#endif /* SHOOTABUG_H */

