#include <qapplication.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qlabel.h>

class MyWidget :public QWidget {
    Q_OBJECT
public:
    MyWidget( QWidget* parent, const char* name = 0 )
        :QWidget( parent, name  ), _active( 0 ) {

        setBackgroundMode( NoBackground );
        QGridLayout* layout = new QGridLayout( this,  4, 4 );
        for ( int i = 0; i < 16; ++i )  {
            QPushButton* but =
                new QPushButton( QString::number(i),  this );
            layout->addWidget( but,  i/4, i%4 );
            connect( but, SIGNAL( clicked() ),
                     this, SLOT( selectButton() ) );
            but->installEventFilter( this );
        }
    }

protected slots:
    void selectButton()  {
        QPushButton* selected = (QPushButton*) sender();
        _active = selected;
        repaint();
    }

    void drawCross( QPixmap *pix ) {
        if ( !_active )
            return;

        QPainter p( pix );
        p.setPen( QPen( black, 3 ) );

        QPoint pos = _active->pos();
        QSize size = _active->size();
        pos += QPoint( size.width()/2, size.height()/2 );

        QPoint p1 = pos + QPoint( -40, -40);
        QPoint p2 = pos + QPoint(  40,  40);
        p.drawLine( p1, p2 );

        p1 = pos + QPoint(  40, -40 );
        p2 = pos + QPoint( -40,  40 );
        p.drawLine( p1, p2 );
    }

protected:
    virtual void paintEvent( QPaintEvent* )  {
        QPixmap pix = _childBuf;
        drawCross( &pix );
        QPainter painter( this, true );
        painter.drawPixmap( 0, 0, pix );
    }

    virtual void resizeEvent( QResizeEvent* e) {
        _childBuf.resize(  e->size() );
        _childBuf.fill( paletteBackgroundColor() );
        QWidget::resizeEvent( e );
    }

    virtual bool eventFilter( QObject* obj, QEvent* e ) {
        if ( e->type() == QEvent::Paint ) {
            static bool block;
            if ( !block ) {
                block = true;
                QWidget* widget = static_cast<QWidget*>( obj );
                QPixmap pix( widget->size() );

                QPainter::redirect( widget, &pix );
                QPaintEvent e( widget->rect() );
                QApplication::sendEvent( widget, &e );
                QPainter::redirect( widget, 0);

                bitBlt( &_childBuf, widget->pos(), &pix );
                repaint();
                block = false;
                return true; // never give the child any paint event itself
            }
        }
        return false;
    }

private:
    QPushButton* _active;
    QPixmap _childBuf;
};

int main( int argc, char** argv ) {
    QApplication app( argc, argv );

    MyWidget* widget = new MyWidget( 0 );
    widget->show();

    QObject::connect( qApp, SIGNAL( lastWindowClosed() ),
		      qApp, SLOT( quit() ) );
    return app.exec();
}

#include "main.moc"
